@@ -2,7 +2,8 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2010 ST-Ericsson AB.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,6 +39,7 @@
 #include "gatresult.h"
 
 #include "atmodem.h"
+#include "vendor.h"
 
 static const char *cgreg_prefix[] = { "+CGREG:", NULL };
 static const char *cgdcont_prefix[] = { "+CGDCONT:", NULL };
@@ -45,6 +47,7 @@ static const char *none_prefix[] = { NULL };
 
 struct gprs_data {
 	GAtChat *chat;
+	unsigned int vendor;
 };
 
 static void at_cgatt_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -85,12 +88,10 @@ error:
 static void at_cgreg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	GAtResultIter iter;
 	ofono_gprs_status_cb_t cb = cbd->cb;
-	int status;
-	const char *str;
-	int lac = -1, ci = -1, tech = -1;
 	struct ofono_error error;
+	int status, lac, ci, tech;
+	struct gprs_data *gd = cbd->user;
 
 	dump_response("at_cgreg_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
@@ -100,33 +101,12 @@ static void at_cgreg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	g_at_result_iter_init(&iter, result);
-
-	if (!g_at_result_iter_next(&iter, "+CGREG:")) {
+	if (at_util_parse_reg(result, "+CGREG:", NULL, &status,
+				&lac, &ci, &tech, gd->vendor) == FALSE) {
 		CALLBACK_WITH_FAILURE(cb, -1, -1, -1, -1, cbd->data);
 		return;
 	}
 
-	/* Skip <n> the unsolicited result code */
-	g_at_result_iter_skip_next(&iter);
-
-	g_at_result_iter_next_number(&iter, &status);
-
-	if (g_at_result_iter_next_string(&iter, &str) == TRUE)
-		lac = strtol(str, NULL, 16);
-	else
-		goto out;
-
-	if (g_at_result_iter_next_string(&iter, &str) == TRUE)
-		ci = strtol(str, NULL, 16);
-	else
-		goto out;
-
-	g_at_result_iter_next_number(&iter, &tech);
-
-out:
-	ofono_debug("cgreg_cb: %d, %d, %d, %d", status, lac, ci, tech);
-
 	cb(&error, status, lac, ci, tech, cbd->data);
 }
 
@@ -140,6 +120,8 @@ static void at_gprs_registration_status(struct ofono_gprs *gprs,
 	if (!cbd)
 		goto error;
 
+	cbd->user = gd;
+
 	if (g_at_chat_send(gd->chat, "AT+CGREG?", cgreg_prefix,
 				at_cgreg_cb, cbd, g_free) > 0)
 		return;
@@ -154,35 +136,15 @@ error:
 static void cgreg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_gprs *gprs = user_data;
-	GAtResultIter iter;
-	gint status, tech = -1;
-	int lac = -1, ci = -1;
-	const char *str;
+	int status, lac, ci, tech;
+	struct gprs_data *gd = ofono_gprs_get_data(gprs);
 
 	dump_response("cgreg_notify", TRUE, result);
 
-	g_at_result_iter_init(&iter, result);
-
-	if (!g_at_result_iter_next(&iter, "+CGREG:"))
+	if (at_util_parse_reg_unsolicited(result, "+CGREG:", &status,
+				&lac, &ci, &tech, gd->vendor) == FALSE)
 		return;
 
-	g_at_result_iter_next_number(&iter, &status);
-
-	if (g_at_result_iter_next_string(&iter, &str))
-		lac = strtol(str, NULL, 16);
-	else
-		goto out;
-
-	if (g_at_result_iter_next_string(&iter, &str))
-		ci = strtol(str, NULL, 16);
-	else
-		goto out;
-
-	g_at_result_iter_next_number(&iter, &tech);
-
-out:
-	ofono_debug("cgreg_notify: %d, %d, %d, %d", status, lac, ci, tech);
-
 	ofono_gprs_status_notify(gprs, status, lac, ci, tech);
 }
 
@@ -213,7 +175,8 @@ static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 	struct gprs_data *gd = ofono_gprs_get_data(gprs);
 
 	g_at_chat_register(gd->chat, "+CGEV:", cgev_notify, FALSE, gprs, NULL);
-	g_at_chat_register(gd->chat, "+CGREG:", cgreg_notify, FALSE, gprs, NULL);
+	g_at_chat_register(gd->chat, "+CGREG:", cgreg_notify,
+				FALSE, gprs, NULL);
 
 	ofono_gprs_register(gprs);
 }
@@ -260,7 +223,13 @@ static void at_cgreg_test_cb(gboolean ok, GAtResult *result,
 
 	g_at_chat_send(gd->chat, cmd, none_prefix, NULL, NULL, NULL);
 	g_at_chat_send(gd->chat, "AT+CGAUTO=0", none_prefix, NULL, NULL, NULL);
-	g_at_chat_send(gd->chat, "AT+CGEREP=2,1", none_prefix,
+
+	/* ST-E modem does not support AT+CGEREP = 2,1 */
+	if (gd->vendor == OFONO_VENDOR_STE)
+		g_at_chat_send(gd->chat, "AT+CGEREP=1,0", none_prefix,
+			gprs_initialized, gprs, NULL);
+	else
+		g_at_chat_send(gd->chat, "AT+CGEREP=2,1", none_prefix,
 			gprs_initialized, gprs, NULL);
 	return;
 
@@ -333,6 +302,7 @@ static int at_gprs_probe(struct ofono_gprs *gprs,
 
 	gd = g_new0(struct gprs_data, 1);
 	gd->chat = chat;
+	gd->vendor = vendor;
 
 	ofono_gprs_set_data(gprs, gd);
 
