@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -53,7 +53,6 @@ static const char *atd_prefix[] = { "+COLP:", NULL };
 
 struct voicecall_data {
 	GSList *calls;
-	unsigned int id_list;
 	unsigned int local_release;
 	unsigned int clcc_source;
 	GAtChat *chat;
@@ -89,10 +88,11 @@ static int class_to_call_type(int cls)
 	}
 }
 
-static struct ofono_call *create_call(struct voicecall_data *d, int type,
+static struct ofono_call *create_call(struct ofono_voicecall *vc, int type,
 					int direction, int status,
 					const char *num, int num_type, int clip)
 {
+	struct voicecall_data *d = ofono_voicecall_get_data(vc);
 	struct ofono_call *call;
 
 	/* Generate a call structure for the waiting call */
@@ -101,7 +101,7 @@ static struct ofono_call *create_call(struct voicecall_data *d, int type,
 	if (!call)
 		return NULL;
 
-	call->id = at_util_alloc_next_id(&d->id_list);
+	call->id = ofono_voicecall_get_next_callid(vc);
 	call->type = type;
 	call->direction = direction;
 	call->status = status;
@@ -131,7 +131,7 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	dump_response("clcc_poll_cb", ok, result);
 
 	if (!ok) {
-		ofono_error("We are polling CLCC and CLCC resulted in an error");
+		ofono_error("We are polling CLCC and received an error");
 		ofono_error("All bets are off for call management");
 		return;
 	}
@@ -160,8 +160,6 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 				ofono_voicecall_disconnected(vc, oc->id,
 								reason, NULL);
 
-			at_util_release_id(&vd->id_list, oc->id);
-
 			o = o->next;
 		} else if (nc && (!oc || (nc->id < oc->id))) {
 			/* new call, signal it */
@@ -179,7 +177,8 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			 */
 			nc->clip_validity = oc->clip_validity;
 
-			if (memcmp(nc, oc, sizeof(struct ofono_call)) && !nc->type)
+			if (memcmp(nc, oc, sizeof(struct ofono_call)) &&
+					!nc->type)
 				ofono_voicecall_notify(vc, nc);
 
 			n = n->next;
@@ -229,8 +228,7 @@ static void generic_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			call = l->data;
 
 			if (req->affected_types & (0x1 << call->status))
-				vd->local_release |=
-					(0x1 << call->id);
+				vd->local_release |= (0x1 << call->id);
 		}
 	}
 
@@ -308,10 +306,10 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	}
 
 	/* Generate a voice call that was just dialed, we guess the ID */
-	call = create_call(vd, 0, 0, 2, num, type, validity);
+	call = create_call(vc, 0, 0, 2, num, type, validity);
 
 	if (!call) {
-		ofono_error("Unable to allocate call, call tracking will fail!");
+		ofono_error("Unable to malloc, call tracking will fail!");
 		return;
 	}
 
@@ -423,66 +421,17 @@ static void at_hangup(struct ofono_voicecall *vc,
 
 static void clcc_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
-	struct cb_data *cbd = user_data;
-	ofono_call_list_cb_t cb = cbd->cb;
-	struct ofono_error error;
-	GSList *calls = NULL;
-	GSList *l;
-	struct ofono_call *list;
-	int num;
-
-	dump_response("clcc_cb", ok, result);
-	decode_at_error(&error, g_at_result_final_response(result));
-
-	if (!ok) {
-		cb(&error, 0, NULL, cbd->data);
-		goto out;
-	}
-
-	calls = at_util_parse_clcc(result);
-
-	if (calls == NULL) {
-		CALLBACK_WITH_FAILURE(cb, 0, NULL, cbd->data);
-		goto out;
-	}
-
-	list = g_try_new0(struct ofono_call, g_slist_length(calls));
-
-	if (!list) {
-		CALLBACK_WITH_FAILURE(cb, 0, NULL, cbd->data);
-		goto out;
-	}
-
-	for (num = 0, l = calls; l; l = l->next, num++)
-		memcpy(&list[num], l->data, sizeof(struct ofono_call));
-
-	cb(&error, num, list, cbd->data);
-
-	g_free(list);
-
-out:
-	g_slist_foreach(calls, (GFunc) g_free, NULL);
-	g_slist_free(calls);
-}
-
-static void at_list_calls(struct ofono_voicecall *vc, ofono_call_list_cb_t cb,
-				void *data)
-{
+	struct ofono_voicecall *vc = user_data;
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
-	struct cb_data *cbd = cb_data_new(cb, data);
+	GSList *l;
 
-	if (!cbd)
-		goto error;
-
-	if (g_at_chat_send(vd->chat, "AT+CLCC", clcc_prefix,
-				clcc_cb, cbd, g_free) > 0)
+	if (!ok)
 		return;
 
-error:
-	if (cbd)
-		g_free(cbd);
+	vd->calls = at_util_parse_clcc(result);
 
-	CALLBACK_WITH_FAILURE(cb, 0, NULL, data);
+	for (l = vd->calls; l; l = l->next)
+		ofono_voicecall_notify(vc, l->data);
 }
 
 static void at_hold_all_active(struct ofono_voicecall *vc,
@@ -650,7 +599,7 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 		return;
 
 	/* Generate an incoming call of unknown type */
-	call = create_call(vd, 9, 1, 4, NULL, 128, 2);
+	call = create_call(vc, 9, 1, 4, NULL, 128, 2);
 
 	if (!call) {
 		ofono_error("Couldn't create call, call management is fubar!");
@@ -705,7 +654,7 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 		type = 9;
 
 	/* Generate an incoming call */
-	call = create_call(vd, type, 1, 4, NULL, 128, 2);
+	call = create_call(vc, type, 1, 4, NULL, 128, 2);
 
 	/* We have a call, and call type but don't know the number and
 	 * must wait for the CLIP to arrive before announcing the call.
@@ -826,11 +775,11 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 
 	ofono_debug("ccwa_notify: %s %d %d %d", num, num_type, cls, validity);
 
-	call = create_call(vd, class_to_call_type(cls), 1, 5,
+	call = create_call(vc, class_to_call_type(cls), 1, 5,
 				num, num_type, validity);
 
 	if (!call) {
-		ofono_error("malloc call structfailed. Call management is fubar");
+		ofono_error("Unable to malloc. Call management is fubar");
 		return;
 	}
 
@@ -896,6 +845,9 @@ static void at_voicecall_initialized(gboolean ok, GAtResult *result,
 	g_at_chat_register(vd->chat, "BUSY", busy_notify, FALSE, vc, NULL);
 
 	ofono_voicecall_register(vc);
+
+	/* Populate the call list */
+	g_at_chat_send(vd->chat, "AT+CLCC", clcc_prefix, clcc_cb, vc, NULL);
 }
 
 static int at_voicecall_probe(struct ofono_voicecall *vc, unsigned int vendor,
@@ -921,6 +873,9 @@ static void at_voicecall_remove(struct ofono_voicecall *vc)
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
+	if (vd->clcc_source)
+		g_source_remove(vd->clcc_source);
+
 	g_slist_foreach(vd->calls, (GFunc) g_free, NULL);
 	g_slist_free(vd->calls);
 
@@ -936,7 +891,6 @@ static struct ofono_voicecall_driver driver = {
 	.dial			= at_dial,
 	.answer			= at_answer,
 	.hangup			= at_hangup,
-	.list_calls		= at_list_calls,
 	.hold_all_active	= at_hold_all_active,
 	.release_all_held	= at_release_all_held,
 	.set_udub		= at_set_udub,
