@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -1003,8 +1003,9 @@ static gboolean decode_status_report(const unsigned char *pdu, int len,
 	if (out->status_report.pi & 0x02) {
 		if (!next_octet(pdu, len, &offset, &out->status_report.dcs))
 			return FALSE;
-	} else
+	} else {
 		out->status_report.dcs = 0;
+	}
 
 	if (out->status_report.pi & 0x04) {
 		int expected;
@@ -1125,8 +1126,9 @@ static gboolean decode_deliver_report(const unsigned char *pdu, int len,
 
 		if (!next_octet(pdu, len, &offset, &octet))
 			return FALSE;
-	} else
+	} else {
 		out->type = SMS_TYPE_DELIVER_REPORT_ACK;
+	}
 
 	pi = octet & 0x07;
 
@@ -1776,8 +1778,9 @@ const char *sms_address_to_string(const struct sms_address *addr)
 				addr->address[0] != '+') {
 		buffer[0] = '+';
 		strcpy(buffer + 1, addr->address);
-	} else
+	} else {
 		strcpy(buffer, addr->address);
+	}
 
 	return buffer;
 }
@@ -3092,8 +3095,9 @@ char *cbs_decode_text(GSList *cbs_list, char *iso639_lang)
 						NULL, 0,
 						(unsigned char *)iso639_lang);
 			iso639_lang[2] = '\0';
-		} else
+		} else {
 			iso639_2_from_language(lang, iso639_lang);
+		}
 	}
 
 	buf = g_new(unsigned char, bufsize);
@@ -3509,7 +3513,7 @@ out:
 	return TRUE;
 }
 
-static GSList *cbs_optimize_ranges(GSList *ranges)
+GSList *cbs_optimize_ranges(GSList *ranges)
 {
 	struct cbs_topic_range *range;
 	unsigned char bitmap[125];
@@ -3666,3 +3670,23 @@ char *cbs_topic_ranges_to_string(GSList *ranges)
 
 	return ret;
 }
+
+static gint cbs_topic_compare(gconstpointer a, gconstpointer b)
+{
+	const struct cbs_topic_range *range = a;
+	unsigned short topic = GPOINTER_TO_UINT(b);
+
+	if (topic >= range->min && topic <= range->max)
+		return 0;
+
+	return 1;
+}
+
+gboolean cbs_topic_in_range(unsigned int topic, GSList *ranges)
+{
+	if (!ranges)
+		return FALSE;
+
+	return g_slist_find_custom(ranges, GUINT_TO_POINTER(topic),
+					cbs_topic_compare) != NULL;
+}
